package com.synnex.phonegap;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.hardware.Camera;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.SurfaceView;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.zxing.BinaryBitmap;
import com.google.zxing.MultiFormatReader;
import com.google.zxing.Result;
import com.google.zxing.client.android.PlanarYUVLuminanceSource;
import com.google.zxing.common.HybridBinarizer;
import com.synnex.barcode.SFHCamera;

public class BarcodeActivity extends Activity
{
	/** Called when the activity is first created. */
	private SurfaceView sfvCamera;
	private SFHCamera sfhCamera;
	private ImageView imgView;
	private View centerView;
	private TextView txtScanResult;
	private Timer mTimer;
	private MyTimerTask mTimerTask;
	// 按照标准HVGA
	final static int width = 480;
	final static int height = 320;
	int dstLeft, dstTop, dstWidth, dstHeight;

	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.barcode);
		this.setTitle("Android条码/二维码识别Demo-----hellogv");
		imgView = (ImageView) this.findViewById(R.id.ImageView01);
		centerView = (View) this.findViewById(R.id.centerView);
		sfvCamera = (SurfaceView) this.findViewById(R.id.sfvCamera);
		sfhCamera = new SFHCamera(sfvCamera.getHolder(), width, height,
				previewCallback);
		txtScanResult = (TextView) this.findViewById(R.id.txtScanResult);
		// 初始化定时器
		mTimer = new Timer();
		mTimerTask = new MyTimerTask();
		mTimer.schedule(mTimerTask, 0, 80);
	}

	class MyTimerTask extends TimerTask
	{
		@Override
		public void run()
		{
			if (dstLeft == 0)
			{// 只赋值一次
				dstLeft = centerView.getLeft() * width
						/ getWindowManager().getDefaultDisplay().getWidth();
				dstTop = centerView.getTop() * height
						/ getWindowManager().getDefaultDisplay().getHeight();
				dstWidth = (centerView.getRight() - centerView.getLeft())
						* width
						/ getWindowManager().getDefaultDisplay().getWidth();
				dstHeight = (centerView.getBottom() - centerView.getTop())
						* height
						/ getWindowManager().getDefaultDisplay().getHeight();
			}
			sfhCamera.AutoFocusAndPreviewCallback();
		}
	}

	/**
	 * 自动对焦后输出图片
	 */
	private Camera.PreviewCallback previewCallback = new Camera.PreviewCallback()
	{
		@Override
		public void onPreviewFrame(byte[] data, Camera arg1)
		{
			// 取得指定范围的帧的数据
			PlanarYUVLuminanceSource source = new PlanarYUVLuminanceSource(
					data, width, height, dstLeft, dstTop, dstWidth, dstHeight, true);
			// 取得灰度图
			Bitmap mBitmap = source.renderCroppedGreyscaleBitmap();
			// 显示灰度图
			imgView.setImageBitmap(mBitmap);
			BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));
			MultiFormatReader reader = new MultiFormatReader();
			try
			{
				Result result = reader.decode(bitmap);
				
				String format = result.getBarcodeFormat().toString();
				String text = result.getText();
				
				String strResult = "BarcodeFormat:" + format + "  text:" + text;
				txtScanResult.setText(strResult);
				if(format != null && format.length() > 0 && text != null && text.length() > 0)
				{
					Log.v("Result", strResult);
					txtScanResult.setText(text);
					stopScaner();
					finishActivity(text);
				}
			} catch (Exception e)
			{
				txtScanResult.setText("Scanning");
			}
		}
	};
	private void finishActivity(String text)
	{
		Intent intent = new Intent();
		Bundle bundle = new Bundle();
		bundle.putString("text", text);
		intent.putExtras(bundle);
		setResult(RESULT_OK, intent);
		finish();
	}
	private void stopScaner()
	{
		mTimerTask.cancel();
		mTimer.cancel();
		sfhCamera.stop();
		sfvCamera.setVisibility(View.INVISIBLE);
	}
	protected void onDestroy()
	{
		stopScaner();
		super.onDestroy();
	}
	public boolean onKeyDown(int keyCode, KeyEvent event)
	{
		if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0)
		{
			stopScaner();
			setResult(RESULT_CANCELED);
		}
		return super.onKeyDown(keyCode, event);
	}
}